module thermo_prop_ig_cplmd

  use vtype_module

  implicit none

  private

  real(dp), parameter :: UnivGasConst = 8.314510d0  ! J mole-1 K-1
  real(dp), parameter :: MolWeight    = 44.0098d-3  ! kg mole-1
  real(dp), parameter :: GasConst     = UnivGasConst / MolWeight ! J kg-1 K-1

  ! Coefficients for specific heat at constant pressure by Lebonnois et al.
  ! (2010)
  real(dp), parameter :: Cp0   = 1.0d3
  real(dp), parameter :: Temp0 = 460.0d0
  real(dp), parameter :: Nu    = 0.35d0

  ! Coefficients below are those with which g/Cp agrees with adiabatic lapse
  ! rate of real gas. (Takahashi et al., 2024)
!!$  real(dp), parameter :: Cp0   = 967.0d0
!!$  real(dp), parameter :: Temp0 = 460.0d0
!!$  real(dp), parameter :: Nu    = 0.30d0


  public :: ThermoPropInit
  public :: ThermoPropInqMolGasConst
  public :: ThermoPropInqMolWeight
  public :: ThermoPropCompFact
  public :: ThermoPropDens
  public :: ThermoPropTemp
  public :: ThermoPropEnthalpy
  public :: ThermoPropCp
  public :: ThermoPropDPressDDensConstTemp
  public :: ThermoPropDPressDTempConstDens
  public :: ThermoPropThermoExpCoef
  public :: ThermoPropAdLapRateDTDP
  public :: ThermoPropTempFromPressEnthalpy
  public :: ThermoPropDensTempFromPressEnthalpy

  !----------------------------------------------------------------------------

contains

  !----------------------------------------------------------------------------
  ! set parameters
  !----------------------------------------------------------------------------

  subroutine ThermoPropInit(     &
!!$    & ConstCp  &
    & )

!!$    real(dp), intent(in), optional :: ConstCp



  end subroutine ThermoPropInit

  !----------------------------------------------------------------------------
  ! inquire molar gas constant
  !----------------------------------------------------------------------------

  function ThermoPropInqMolGasConst() &
    & result( Val )

    real(dp) :: Val

    Val = UnivGasConst

  end function ThermoPropInqMolGasConst

  !----------------------------------------------------------------------------
  ! inquire molecular weight
  !----------------------------------------------------------------------------

  function ThermoPropInqMolWeight() &
    & result( Val )

    real(dp) :: Val

    Val = MolWeight

  end function ThermoPropInqMolWeight

  !----------------------------------------------------------------------------
  ! compressibility factor
  !----------------------------------------------------------------------------

  function ThermoPropCompFact( Dens, Temp ) &
    & result( CompFact )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: CompFact


    ! Local variables
    !


    CompFact = 1.0d0


  end function ThermoPropCompFact

  !----------------------------------------------------------------------------
  ! density
  !----------------------------------------------------------------------------

  function ThermoPropDens( Press, Temp ) &
    & result( Dens )

    real(dp), intent(in) :: Press
    real(dp), intent(in) :: Temp
    real(dp)             :: Dens


    ! Local variables
    !


    Dens = Press / ( GasConst * Temp )


  end function ThermoPropDens

  !----------------------------------------------------------------------------
  ! temperature
  !----------------------------------------------------------------------------

  function ThermoPropTemp( Press, Dens ) &
    & result( Temp )

    real(dp), intent(in) :: Press
    real(dp), intent(in) :: Dens
    real(dp)             :: Temp


    ! Local variables
    !


    Temp = Press / ( Dens * GasConst )


  end function ThermoPropTemp

  !----------------------------------------------------------------------------
  ! enthalpy
  !----------------------------------------------------------------------------

  function ThermoPropEnthalpy( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !


    Val = Cp0 * Temp0 / ( Nu + 1.0d0 ) * ( Temp / Temp0 )**(Nu+1.0d0)



  end function ThermoPropEnthalpy

  !----------------------------------------------------------------------------
  ! isobaric heat capacity (specific heat on constant pressure)
  !----------------------------------------------------------------------------

  function ThermoPropCp( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !


    Val = Cp0 * ( Temp / Temp0 )**Nu


  end function ThermoPropCp

  !----------------------------------------------------------------------------
  ! DPDRho
  !----------------------------------------------------------------------------

  function ThermoPropDPressDDensConstTemp( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Delta
    real(dp) :: Tau


    Val = GasConst * Temp


  end function ThermoPropDPressDDensConstTemp

  !----------------------------------------------------------------------------
  ! DPDT
  !----------------------------------------------------------------------------

  function ThermoPropDPressDTempConstDens( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !


    Val = Dens * GasConst


  end function ThermoPropDPressDTempConstDens

  !----------------------------------------------------------------------------
  ! DHDRho
  !----------------------------------------------------------------------------

  function ThermoPropDEnthalpyDDensConstTemp( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !


    Val = 0.0d0


  end function ThermoPropDEnthalpyDDensConstTemp

  !----------------------------------------------------------------------------
  ! DHDT
  !----------------------------------------------------------------------------

  function ThermoPropDEnthalpyDTempConstDens( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !


    Val = ThermoPropCp( Dens, Temp )


  end function ThermoPropDEnthalpyDTempConstDens

  !----------------------------------------------------------------------------
  ! thermal expansion coefficient
  !----------------------------------------------------------------------------

  function ThermoPropThermoExpCoef( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !

    Val = 1.0d0 / Temp


  end function ThermoPropThermoExpCoef

  !----------------------------------------------------------------------------
  ! thermal expansion coefficient
  !----------------------------------------------------------------------------

  function ThermoPropAdLapRateDTDP( Dens, Temp ) &
    & result( Val )

    real(dp), intent(in) :: Dens
    real(dp), intent(in) :: Temp
    real(dp)             :: Val


    ! Local variables
    !
    real(dp) :: Cp


    Cp = ThermoPropCp( Dens, Temp )

    Val = 1.0d0 / ( Cp * Dens )


  end function ThermoPropAdLapRateDTDP

  !----------------------------------------------------------------------------
  ! temperature from pressure and enthalpy
  !----------------------------------------------------------------------------

  function ThermoPropTempFromPressEnthalpy( &
    & Press, Enthalpy & ! (in)
    & ) &
    & result( Val )

    real(dp), intent(in ) :: Press
    real(dp), intent(in ) :: Enthalpy

    real(dp) :: Val


    ! Local variables
    !


    Val = Temp0 * ( Enthalpy / ( Cp0 * Temp0 / ( Nu + 1.0d0 ) ) )**(1.0d0/(Nu+1.0d0))


  end function ThermoPropTempFromPressEnthalpy

  !----------------------------------------------------------------------------
  ! temperature and density from pressure and enthalpy
  !----------------------------------------------------------------------------

  subroutine ThermoPropDensTempFromPressEnthalpy( &
    & Press, Enthalpy, & ! (in)
    & Dens, Temp       & ! (out)
    & )

    real(dp), intent(in ) :: Press
    real(dp), intent(in ) :: Enthalpy
    real(dp), intent(out) :: Dens
    real(dp), intent(out) :: Temp


    ! Local variables
    !


    Temp = Temp0 * ( Enthalpy / ( Cp0 * Temp0 / ( Nu + 1.0d0 ) ) )**(1.0d0/(Nu+1.0d0))
    Dens = Press / ( GasConst * Temp )


  end subroutine ThermoPropDensTempFromPressEnthalpy

  !----------------------------------------------------------------------------

end module thermo_prop_ig_cplmd
